#!/bin/csh
#
# script to replace ESO Hierarchical Format image header keywords by
# standard FITS compliant header keywords.    R.A. Jansen -- Dec 4, 2001
# Arguments 2--4: 1=image name, 2=keyword translation table, 3=quit-on-
# error switch and 4=verbosity switch.

if ( $#argv < 2 || $#argv > 4 ) then
    echo "Usage: eso2fits.csh <imheader> <transtab> [-q] [-v]"
    exit
endif

# Read mandatory input parameters...
set imheader = $1
set transtab = $2
set tmphead = /tmp/$imheader
shift; shift

# Check for optional switches...
set verb = 0
set quit = 0
while ( a == a )
    if ( "-v" == "$1" ) then
	set verb = 1 ; shift
    else if ( "-q" == "$1" ) then
	set quit = 1 ; shift
    else if ( "-v" != "$1" && "-q" != "$1" && "" != "$1" ) then
	echo "ERROR: unrecognized option "$1
	echo "Usage: eso2fits.csh <imheader> <transtab> [-q] [-v]"
	exit
    else
	break
    endif
end

# Check whether translation table and image header exist and are
# read- and read-write- accessible...
if ( ! -r $transtab ) then
    echo "Cannot not find or access "$transtab" for reading!"
    exit
endif
if ( ! -r $imheader || ! -w $imheader ) then
    echo "Cannot not find or access "$imheader" for reading and writing!"
    exit
endif

# Remove temporary file if it already exists...
if ( -r $tmphead ) then
    \rm -fr $tmphead
endif

# Count the number of lines in input files...
set nt = `cat $transtab | wc -l`
set nh = `cat $imheader | wc -l`

# Opening message...
if ( $verb == 1 ) then
   echo "eso2fits.csh: "${user}"@"`uname -n`"   "`date`
   echo "  image header= "$imheader"        "$nh" lines"
   echo "  transl.table= "$transtab"        "$nt" lines"
   echo "  quit-on-err = "$quit"        verbose     = "$verb
endif

# Create a nt-element string-valued dummy vector...
set it = 0
set vdum = ""
set elem = " a"
while ( $it < $nt )
    @ it = $it + 1
    set vdum = "$vdum$elem"
end
# Create a nh-element string-valued dummy vector...
set ih = 0
set wdum = ""
while ( $ih < $nh )
    @ ih = $ih + 1
    set wdum = "$wdum$elem"
end

# Read the translation table into memory...
echo -n "Reading translation table into memory"
set esokey = ( $vdum )
set fitkey = ( $vdum )
set fitcom = ( $vdum )
set it = 0
set hsh = 0
while ( $it < $nt )
    @ it = $it + 1
    @ hsh = $it % 10
    #if ( $verb == 1 && $hsh == 0 ) echo -n "..."$it    
    if ( $hsh == 0 ) echo -n "."
    set esokey[$it]="`tail +$it $transtab | head -1 | awk -F! '{print "'$1'"}'`"
    set fitkey[$it]="`tail +$it $transtab | head -1 | awk -F! '{print "'$2'"}'`"
    set fitcom[$it]="`tail +$it $transtab | head -1 | awk -F! '{print "'$3'"}'`"
end
#if ( $verb == 1 ) echo " "
echo "done"

# Read the image header into memory...
echo -n "Reading image header into memory"
set hstr = ( $wdum )
set hkey = ( $wdum )
set hval = ( $wdum )
set hcom = ( $wdum )
set ih = 5
set hsh = 0
while ( $ih < $nh )
    @ ih = $ih + 1
    @ hsh = $ih % 10
    #if ( $verb == 1 && $hsh == 0 ) echo -n "..."$ih
    if ( $hsh == 0 ) echo -n "."
    set hstr[$ih]="`tail +$ih $imheader | head -1`"
    set hkey[$ih]="`tail +$ih $imheader | head -1 | awk -F= '{print "'$1'"}'`"
    set htmp="`tail +$ih $imheader | head -1 | awk -F= '{print "'$2'"}' | awk -F'/ ' '{print "'$1'"}'`"
    set hcom[$ih]="`tail +$ih $imheader | head -1 | awk -F'/ ' '{print "'$2'"}'`"

    # Padd values to at least 22 characters (FITS standard)...
    set nc = `echo "$htmp" | wc | awk '{print $3}'`
    if ( $nc < 23 ) then
        set ic = $nc
        if ( `echo "$htmp" | cut -c2-2` == "'" ) then
            while ( $ic < 23 )
                @ ic = $ic + 1
                set htmp = "$htmp"" "
            end
        else
            while ( $ic < 23 )
                @ ic = $ic + 1
                set htmp = " ""$htmp"
            end
	endif
    endif
    # Check for excessive trailing spaces...
    set nc = `echo "$htmp" | wc | awk '{print $3}'`
    @ nc = $nc - 1
    if ( $nc > 23 ) then
	@ ic = $nc - 1
 	while ( $ic > 22 )
            set cc = "`echo "'"$htmp"'" | cut -c$ic-$ic`"
            if ( "$cc" != " " ) break
	    @ ic = $ic - 1
	end
	set tmph = "`echo "'"$htmp"'" | cut -c1-$ic`"
	set htmp = "$tmph"
    endif
    set hval[$ih] = "$htmp"

end
#if ( $verb == 1 ) echo " "
echo "done"

# Assume the image header was dumped to ASCII using IRAF 'imheader'...
head -5 $imheader > $tmphead

# Loop through header one record at a time and if the keyword starts
# with 'HIERARCH', loop through the translation table one line at a
# time and check for match...
if ( $verb == 0 ) then
    echo -n "Translating"
else
    echo "Starting translation:"
endif
set ih=5
while ( $ih < $nh )
    @ ih = $ih + 1
    @ hsh = $ih % 10
    if ( $verb == 0 && $hsh == 0 ) echo -n "."

    # Check whether header record is in ESO Hierarchical Format...
    if ( `echo "$hkey[$ih]" | cut -c1-8` == "HIERARCH" ) then
        set it = 0
	set match = 0
        while ( $it < $nt )
	    @ it = $it + 1
            if ( "$esokey[$it]" == "$hkey[$ih]" ) then
		set match = 1
		set hstr[$ih] = "$fitkey[$it]""=""$hval[$ih]""/ ""$fitcom[$it]"
		echo "$hstr[$ih]" | cut -c1-80 >> $tmphead

		# Progress report...
		if ( $verb == 1 ) then
		    echo "#"$ih": ""$esokey[$it]"" --> ""$fitkey[$it]""=""$hval[$ih]""/"
		endif

		break
	    endif
        end
	# What to do on error (i.e. no translation found)?
	if ( $match == 0 ) then
	    if ( quit == 1 ) then
		echo "ERROR: found no match for ""$hkey[$ih]"" in "$transtab"!"
		exit
	    else
		echo "WARNING: found no match for ""$hkey[$ih]"" in "$transtab"!"
		echo "$hstr[$ih]" | cut -c1-80 >> $tmphead

		# Progress report...
		if ( $verb == 1 ) echo "#"$ih": ""$hstr[$ih]""/"
	    endif
	endif

    else if ( `echo "$hkey[$ih]" | cut -c1-7` == "COMMENT" ) then
	echo "$hstr[$ih]" | cut -c1-80 >> $tmphead

	## Progress report...
	#if ( $verb == 1 ) echo "#"$ih": ""$hstr[$ih]""/"

    else
	# Header record conforms to FITS standard...
	set hstr[$ih] = "$hkey[$ih]""=""$hval[$ih]""/ ""$hcom[$ih]"
	echo "$hstr[$ih]" | cut -c1-80 >> $tmphead

	## Progress report...
	#if ( $verb == 1 ) then
	#    echo "#"$ih": ""$hkey[$ih]""=""$hval[$ih]""/"
	#endif

    endif
end
if ( $verb == 0 ) then
   echo "done"
else
   echo " "
endif

# Repair the anomalous "'/" (missing space between quote and slash) and
# replace the original with the converted header...
sed 's:'"'/:'"' /:g' $tmphead > $imheader
\rm -fr $tmphead

# Closing message...
if ( $verb == 1 ) then
   echo "eso2fits.csh -- Done."
endif

